import json
import random
import requests
from .base_class import *
from urllib3.util.retry import Retry
from requests.adapters import HTTPAdapter

us = config_dict['base_ua']

# url 分析
import re


class SpiderUrlType:
    """
    解析用户传入的 URL，返回平台、类型、work_url 和 work_id
    平台:抖音,小红书
    类型:视频,用户,笔记
    return {
            "platform": "douyin",  如果platform="" 代表获取实拍
            "work_type": "user",
            "work_url": "http_xxxxxxxxxx",
            "work_id": "xxxxxx"
        }
    """

    def __init__(self, spider_url, url_type='video'):
        self.url_type = url_type
        self.spider_url = spider_url.strip()
        self.result = {
            "platform": "",
            "work_type": "",
            "work_url": "",
            "work_id": ""
        }
        if 'http' not in self.spider_url:
            return
        self._parse()

    def _parse(self):
        url = self.spider_url

        if 'douyin.com' in url:
            self.result['platform'] = 'douyin'
            self._parse_douyin(url)
        elif 'xiaohongshu.com' in url or 'xhslink.com' in url:
            self.result['platform'] = 'xhs'
            self._parse_xhs(url)
        elif 'weixin.' in url:
            self.result['platform'] = 'wx'
            self._parse_wechat(url)
        else:
            self.result['platform'] = ''

    def _parse_douyin(self, url):
        if self.url_type == 'video':
            # https://www.douyin.com/video/7212458016895292672
            match = re.search(r'/video/(\d+)', url)
            if match:
                self.result['work_type'] = 'video'
                self.result['work_id'] = match.group(1)
                self.result['work_url'] = f"https://www.douyin.com/video/{self.result['work_id']}"
        elif self.url_type == 'user':
            sec_uid = None

            # https://www.douyin.com/user/MS4wLjABAAAA6F3tMONSZUMg6fkxAyJP-bXtf1VlHsPFMq_ORQ1-zhE?from_tab_name=main
            if 'www.douyin.com' in url and 'MS4' in url:
                sec_uid = url.split('https://www.douyin.com/user/')[-1].split('?')[0]

            elif '://v.douyin.com' in url:
                url_pattern = r'://v\.douyin\.com/\w+/'
                matches = re.findall(url_pattern, url)
                if matches:
                    res = HttpJike.get(url=f'https{matches[0]}').ret_url
                    sec_uid = res.split('https://www.iesdouyin.com/share/user/')[-1].split('?')[0]

            if sec_uid:
                self.result['work_type'] = 'user'
                self.result['work_id'] = sec_uid
                self.result['work_url'] = f"https://www.douyin.com/user/{sec_uid}"

    def _parse_xhs(self, url):
        # 示例：https://www.xiaohongshu.com/explore/5f73e5f30000000001001a7a
        if '/explore/' in url:
            match = re.search(r'/explore/([a-zA-Z0-9]+)', url)
            if match:
                self.result['work_type'] = 'note'
                self.result['work_id'] = match.group(1)
                self.result['work_url'] = f"https://www.xiaohongshu.com/explore/{self.result['work_id']}"
        elif '/user/profile/' in url:
            match = re.search(r'/user/profile/([a-zA-Z0-9]+)', url)
            if match:
                self.result['work_type'] = 'user'
                self.result['work_id'] = match.group(1)
                self.result['work_url'] = f"https://www.xiaohongshu.com/user/profile/{self.result['work_id']}"

    def _parse_wechat(self, url):
        # 示例：https://mp.weixin.qq.com/s?__biz=MzA5NDY1NjMyMQ==&mid=2657502275&idx=1
        self.result['work_type'] = 'article'
        self.result['work_url'] = url
        biz_match = re.search(r'__biz=([a-zA-Z0-9+=]+)', url)
        if biz_match:
            self.result['work_id'] = biz_match.group(1)

    def get_result(self):
        return self.result


# requests 封装
class HttpJike(object):
    us = us

    def __init__(self):
        self.status_code = 500
        self.msg = 'ok'
        self.text = None
        self.json = None
        self.ret_url = None

    # cookie 分隔
    @staticmethod
    def cookie_format(cookie):
        cookie_dict = {}
        c = cookie.split(";")
        for i in c:
            cc = i.split('=')
            if len(cc) > 1:
                cookie_dict[str(cc[0]).strip()] = str(cc[1]).strip()
            else:
                cookie_dict[str(cc[0]).strip()] = ''
        return cookie_dict

    # ip代理 隧道代理
    @staticmethod
    def proxies_choose(p=1, httpx=0):
        # 注意:目前只有 1,2 两个可以使用  httpx特殊请求
        if p is None:
            p = random.randint(1, 2)

        proxy = config_dict["proxy"]["tunnel"][f"proxy_{p}"]['proxy']
        port = config_dict["proxy"]["tunnel"][f"proxy_{p}"]['port']
        acc = config_dict["proxy"]["tunnel"][f"proxy_{p}"]['acc']
        pwd = config_dict["proxy"]["tunnel"][f"proxy_{p}"]['pwd']

        proxies = {
            "http": f"http://{acc}:{pwd}@{proxy}:{port}/",
            "https": f"http://{acc}:{pwd}@{proxy}:{port}/"
        }
        if httpx == 1:
            proxies = {
                "http://": f"http://{acc}:{pwd}@{proxy}:{port}/",
                "https://": f"http://{acc}:{pwd}@{proxy}:{port}/"
            }
        return proxies

    # scrapy 代理选择 数据返回
    @classmethod
    def proxies_choose_dict(cls, p):
        # 注意:目前只有 1,2,3 两个可以使用
        proxies_dict = {
            'proxy': config_dict["proxy"]["tunnel"][f"proxy_{p}"]['proxy'],
            'port': config_dict["proxy"]["tunnel"][f"proxy_{p}"]['port'],
            'acc': config_dict["proxy"]["tunnel"][f"proxy_{p}"]['acc'],
            'pwd': config_dict["proxy"]["tunnel"][f"proxy_{p}"]['pwd']
        }
        return proxies_dict

    # 异步代理 的使用
    @staticmethod
    def aiohttp_proxy():
        ret = []
        ip_tunnel = config_dict['proxy']['tunnel']
        for i in ip_tunnel:
            ret.append({
                'proxy': f'http://{ip_tunnel[i]["proxy"]}:15818',
                'a': ip_tunnel[i]['acc'],
                'p': ip_tunnel[i]['pwd'],
            })
        return ret

    @staticmethod
    def get_headers(headers):
        if headers is None:
            return config_dict['base_headers']
        return headers

    @classmethod
    def get(cls, url, headers=None, proxies=None):
        req = cls()
        try:
            response = requests.get(
                url=url,
                headers=cls.get_headers(headers=headers),
                proxies=proxies
            )
            req.status_code = response.status_code
            req.ret_url = response.url
            req.text = response.text
            req.json = response.json()

            if response.status_code != 200:
                req.msg = '状态码错误'
        except Exception as e:
            req.msg = f'err {e}'
        return req

    @classmethod
    def post(cls, url, headers=None, data=None):
        req = cls()
        try:
            response = requests.post(
                url=url,
                headers=cls.get_headers(headers=headers),
                data=json.dumps(data),
            )
            req.status_code = response.status_code
            req.text = response.text
            req.json = response.json()

            if response.status_code != 200:
                req.msg = '状态码错误'
        except Exception as e:
            req.msg = f'err {e}'
        return req

    # 代理
    @classmethod
    def http_ip(cls, ip):
        proxies = {
            'https': ip,
            'http': ip
        }
        return proxies

    # 返回一个http代理
    @classmethod
    def http_proxy(cls):
        return {'https': '49.70.176.21:31919', 'http': '49.70.176.21:31919'}

    @classmethod
    def params_link(cls, url, params):
        return f"{url}?" f"{'&'.join([f'{k}={v}' for k, v in params.items()])}"

    @classmethod
    def base_headers(cls):
        try:
            return config_dict['base_headers']
        except:
            return {
                'user-agent': 'Mozilla/5.0 (Windows NT 10.0; Win64; x64) AppleWebKit/537.36 (KHTML, like Gecko) Chrome/99.0.4844.74 Safari/537.36 Edg/99.0.1150.46'}

    @staticmethod
    def scrapy_simple_sitting(ts=0.1, tt=8, log=False, cookie=True):
        def_sitting = {
            "LOG_ENABLED": log,  # 日志开启
            "HTTPERROR_ALLOWED_CODES": [i for i in range(999)],  # 允许所有 HTTP 错误码
            "REDIRECT_ENABLED": False,  # 禁用重定向
            "DOWNLOAD_DELAY": ts,  # 每次请求间隔 1 秒
            "CONCURRENT_REQUESTS": tt,  # 最大并发请求数
            "DOWNLOADER_MIDDLEWARES": {
                'scrapy.downloadermiddlewares.httpproxy.HttpProxyMiddleware': 1,  # 启用代理中间件
            }
        }
        if cookie:
            def_sitting['COOKIES_ENABLED'] = False
        return def_sitting

    @staticmethod
    def requests_retry_session(
            retries=3,
            backoff_factor=0.5,
            status_forcelist=(500, 502, 503, 504),
            session=None,
    ):
        session = session or requests.Session()
        retry = Retry(
            total=retries,
            read=retries,
            connect=retries,
            backoff_factor=backoff_factor,
            status_forcelist=status_forcelist,
            raise_on_status=False,
        )
        adapter = HTTPAdapter(max_retries=retry)
        session.mount('http://', adapter)
        session.mount('https://', adapter)
        return session

    @staticmethod
    def spider_url_type(spider_url, url_type='video'):
        return SpiderUrlType(spider_url, url_type=url_type).get_result()



import time
import psycopg2
from psycopg2 import pool
from .cache_class import *  # 复用您的缓存系统


class PgsqlDb:
    _pool = None

    @staticmethod
    def db_pgsql(path=None, ret_config=None):
        config_pgsql = config_dict["pgsql"]  # 从缓存获取配置
        if path == 1:
            db_cfg = "pgsql_in"
        elif path == 2:
            db_cfg = "pgsql_test"
        elif path == 3:
            db_cfg = "pgsql_loc"
        else:
            db_cfg = "pgsql_out"

        # 寻找特定的数据库配置
        if path in config_pgsql:
            db_cfg = path

        pgsql_host = config_pgsql[db_cfg]['host']
        pgsql_user = config_pgsql[db_cfg]['user']
        pgsql_passwd = config_pgsql[db_cfg]['pwd']
        pgsql_db = config_pgsql[db_cfg]['db']
        pgsql_port = int(config_pgsql[db_cfg]['port'])

        if ret_config:
            return {
                'host': pgsql_host,
                'user': pgsql_user,
                'password': pgsql_passwd,
                'database': pgsql_db,
                'port': pgsql_port
            }

        # 使用连接池
        if PgsqlDb._pool is None:
            PgsqlDb._pool = pool.SimpleConnectionPool(
                minconn=1,
                maxconn=10,
                host=pgsql_host,
                user=pgsql_user,
                password=pgsql_passwd,
                database=pgsql_db,
                port=pgsql_port
            )
        return PgsqlDb._pool.getconn()

    def pgsql_db(self, method, table, conn_tp=None, **kwargs):
        """
        method
            - s -- select
            - up -- update_more_byid
            - ins -- insert
            - iss -- insert_all
            - tc -- create_table
            - te -- table_exist
            - sql -- 执行sql
        """
        if conn_tp is None:
            conn_tp = 0
        sql = kwargs.get('sql', '')
        save_data = kwargs.get('save_data')
        # 其他链接
        conn_other = kwargs.get('conn_other')
        if conn_other:
            conn = conn_other
        else:
            conn = self.db_pgsql(path=conn_tp)
        # 通用sql 查看表是否存在
        sql_table_exist = f"SELECT EXISTS (SELECT FROM information_schema.tables WHERE table_name = '{table}')"
        try:
            with conn.cursor() as cursor:
                if method == 'insert' or method == 'ins':
                    save_data = kwargs['save_data']
                    columns = ', '.join(save_data.keys())
                    placeholders = ', '.join(['%s'] * len(save_data))
                    params = tuple(save_data.values())
                    sql = f"INSERT INTO {table} ({columns}) VALUES ({placeholders})"
                    cursor.execute(sql, params)
                    conn.commit()
                elif method == 'insert_all' or method == 'iss':
                    fields = list(save_data[0].keys())
                    placeholders = ', '.join(f'%({i})s' for i in fields)
                    fields_str = ','.join(fields)
                    sql_inserts = f"INSERT INTO {table} ({fields_str}) values({placeholders})"
                    n = cursor.executemany(sql_inserts, save_data)
                    conn.commit()
                    return n
                elif method == 'create_table' or method == 'tc':
                    table_exist = cursor.execute(sql_table_exist)
                    if table_exist:
                        del_and_create = kwargs.get('del_and_create', 0)
                        if del_and_create:
                            print('表已经存在 删除并创建')
                            self.pgsql_db(method='dt', table=table, conn_tp=conn_tp)
                        else:
                            print('表已经存在')
                            return '表已经存在'
                    fields_sql = []
                    field_cfg = kwargs['field_cfg']
                    # 直接使用 field_cfg['id'] 作为主键定义
                    fields_sql.append(field_cfg['id'])
                    # 处理其他字段
                    for f in field_cfg['fields']:
                        name = f[0]
                        field_type = f[1]
                        length = f[2]
                        default = f[3]
                        comment = f[4]
                        # 修改字符串默认值的处理
                        if field_type == 'TIMESTAMP':
                            fields_sql.append(f"{name} {field_type} WITH TIME ZONE DEFAULT {default}")
                        elif field_type == 'VARCHAR':
                            fields_sql.append(f"{name} {field_type}({length}) DEFAULT '{default}'")
                        elif field_type == 'INT':
                            fields_sql.append(f"{name} INTEGER DEFAULT {default}")
                        elif field_type == 'BIGINT':
                            fields_sql.append(f"{name} BIGINT DEFAULT {default}")
                        elif field_type == 'SMALLINT':
                            fields_sql.append(f"{name} SMALLINT DEFAULT {default}")
                        elif field_type == 'FLOAT':
                            fields_sql.append(f"{name} FLOAT DEFAULT {default}")
                        elif field_type == 'DOUBLE':
                            fields_sql.append(f"{name} DOUBLE PRECISION DEFAULT {default}")
                        elif field_type == 'DECIMAL':
                            fields_sql.append(f"{name} DECIMAL({length}) DEFAULT {default}")
                        elif field_type == 'TEXT':
                            fields_sql.append(f"{name} TEXT DEFAULT '{default}'")
                        elif field_type == 'CHAR':
                            fields_sql.append(f"{name} CHAR({length}) DEFAULT '{default}'")
                        elif field_type == 'BOOLEAN':
                            fields_sql.append(f"{name} BOOLEAN DEFAULT {default}")
                        elif field_type == 'JSON':
                            fields_sql.append(f"{name} JSONB DEFAULT NULL")
                        elif field_type == 'ARRAY':
                            fields_sql.append(f"{name} TEXT[] DEFAULT ARRAY[]::TEXT[]")
                        elif field_type == 'DATE':
                            fields_sql.append(f"{name} DATE DEFAULT {default}")
                        elif field_type == 'TIME':
                            fields_sql.append(f"{name} TIME DEFAULT {default}")
                        elif field_type == 'UUID':
                            fields_sql.append(f"{name} UUID DEFAULT gen_random_uuid()")
                        elif field_type == 'BYTEA':
                            fields_sql.append(f"{name} BYTEA DEFAULT NULL")
                        elif field_type == 'ENUM':
                            # 修改这里，使用单引号
                            fields_sql.append(f"{name} TEXT DEFAULT '{default}'")
                        elif field_type == 'MONEY':
                            fields_sql.append(f"{name} MONEY DEFAULT {default}")
                        elif field_type == 'INET':
                            fields_sql.append(f"{name} INET DEFAULT NULL")
                        elif field_type == 'CIDR':
                            fields_sql.append(f"{name} CIDR DEFAULT NULL")
                        else:
                            fields_sql.append(f"{name} {field_type} DEFAULT {default}")
                    if fields_sql:
                        this_time = time.strftime("%Y-%m-%d %X", time.localtime(int(time.time())))
                        table_notes = f'{this_time} 【高阳】创建此表'
                        # 打印SQL语句以便调试
                        sql_create_base = f"""
                            CREATE TABLE {table} (
                                {','.join(fields_sql)}
                            );
                            COMMENT ON TABLE {table} IS '{table_notes}';
                        """
                        print("创建表SQL:", sql_create_base)  # 添加调试输出
                        cursor.execute(sql_create_base)
                        # 增加唯一索引
                        field_index = field_cfg['field_index']
                        if field_index:
                            index_name = f"idx_{table}_{'_'.join(field_index)}"
                            if len(field_index) == 1:
                                sql_index = f"CREATE UNIQUE INDEX {index_name} ON {table} ({field_index[0]});"
                            else:
                                sql_index = f"CREATE UNIQUE INDEX {index_name} ON {table} ({','.join(field_index)});"
                            print("创建索引SQL:", sql_index)  # 添加调试输出
                            cursor.execute(sql_index)
                        conn.commit()
                        print(f"创建{table}成功")
                        return f"创建{table}成功"
                elif method == 'select' or method == 's':
                    cursor.execute(sql)
                    return cursor.fetchall()
                elif method == 'update_more_byid' or method == 'up':
                    if save_data:
                        fields = list(save_data[0].keys())  # 假设 save_data 至少有一个元素
                        update_fields = [f'{i}=%s' for i in fields[:-1]]
                        # 使用最后一个字段作为 WHERE 条件
                        where_field = fields[-1]
                        sql_update = f"UPDATE {table} SET {','.join(update_fields)} WHERE {where_field} = %s"

                        # 检查 save_data 是单个记录还是多条记录
                        if isinstance(save_data, list):
                            # 批量更新
                            tuple_data_list = [tuple(data.values()) for data in save_data]
                            cursor.executemany(sql_update, tuple_data_list)
                            conn.commit()
                            # 可以返回更新的记录数，如果需要的话
                            # return len(tuple_data_list)
                        else:
                            # 单条更新
                            params = tuple(save_data.values())
                            cursor.execute(sql_update, params)
                            conn.commit()
                    else:
                        print("更新操作需要提供 save_data")
                elif method == 'sql':
                    cursor.execute(sql)
                    conn.commit()

        except Exception as e:
            print(f"数据库链接错误:{e}")
            conn.rollback()
        finally:
            if not conn_other:  # 如果不是外部传入的连接，则归还到连接池
                PgsqlDb._pool.putconn(conn)
